# CAOS with Cloud Application Integration

[CAOS](https://kb.wisc.edu/registrar/page.php?id=11753) (Curricular and Academic Web Services) is a SOAP API, organized primarily by curricular data. It also sits behind a firewall and is only accessible if your IP has been added to allow list. Our Informatica Secure Agents have been added to allow list, however the Informatica CAI service lives in the cloud, hosted by Informatica and cannot be. While this doesn't prevent us from consuming CAOS data from CAI, it adds a few steps to our process.

Another thing to consider is that setting up an integration to [consume CAOS data is much easier in CDI](docs/tutorials/API_Related/CAOS_SOAP_API/CAOS_with_CDI.md). If you are receiving real-time updates with Webhooks or another messaging service through CAI, you can add a call to a CDI Taskflow or Mapping Task within the CAI Process. In order to do this, you will need to add the [CAI Anonymous User](https://docs.informatica.com/cloud-common-services/administrator/current-version/user-administration/users/application-integration-anonymous-user.html) to the permissions on your CDI objects.

**We recommend that this CAI route be explored only by experienced IICS users or with close consultation with the [Integration Team](mailto:integration-platform@doit.wisc.edu).**

## Cloud Application Integration

The steps for setting up a Process in CAI are as follows:

1) Create a Service Connector.
    - For CAOS, you will need to download the .wsdl file instead of refer to the URL, due to the URL only being accessible if your IP is added to allow list.
2) Create an App Connection and refer it to the Service Connector created in step 1.
3) Create a Process and add a step within it that uses the App Connection that refers to the Service Connector.

![CAI_Process](/images/CAI_Process.png)

For more in-depth understanding of this process, please read through section 8 of the [Canvas Tutorial](https://canvas.wisc.edu/courses/378847) we created for general instruction on CAI.

### Specific for CAOS data requests

1) As stated above, the .wsdl will need to be in the form of a file instead of URL to create the Service Connector.
2) The Start Properties of your Process will need to have the "Run On" setting set to the Secure Agent server you typically work with.

![RunOnSA](/images/RunOnSA.png)

Once you have gotten this far, in order to extract the data and assign it to values for writing to a file, you will need to be familiar with XQuery. This is the language that the Expression Editor recognizes in the Assignment steps. Below is an example of taking a CAOS GetStudents response and assigning data elements. These assignments are later plugged into a File Writer step to write to a .csv.

![ExpressionCAI](/images/ExpressionCAI.png)

## References

* [Canvas Course](https://canvas.wisc.edu/courses/378847) for general IICS training. Section 8 is focused on CAI.
* [CAOS User Guide](https://git.doit.wisc.edu/road/documentation/-/blob/main/caos/caos-user-guide.md)
* [Legacy CAOS Wiki](https://git.doit.wisc.edu/road/caos/legacy-wiki), migrated to preserve the old Atlassian Wiki pages, but is not as user-friendly as the User Guide.